@@ -114,6 +114,7 @@
 #include <QSortFilterProxyModel>
 #include <QFileSystemModel>
 #include <QFileInfo>
+#include <QResource>
 #include <Qt>
 #include <QDebug>
 #include <algorithm>
@@ -382,7 +383,7 @@ MainWindow::switchToNewProject(
 	if (out_dir.isEmpty()) {
 		m_ptrThumbnailCache.reset();
 	} else {
-		m_ptrThumbnailCache = createThumbnailCache();
+		m_ptrThumbnailCache = Utils::createThumbnailCache(m_outFileNameGen.outDir());
 	}
 	resetThumbSequence(currentPageOrderProvider());
 
@@ -392,18 +393,6 @@ MainWindow::switchToNewProject(
 	updateMainArea();
 }
 
-IntrusivePtr<ThumbnailPixmapCache>
-MainWindow::createThumbnailCache()
-{
-	QSize const max_pixmap_size(200, 200);
-	
-	return IntrusivePtr<ThumbnailPixmapCache>(
-		new ThumbnailPixmapCache(
-			m_outFileNameGen.outDir()+"/cache/thumbs", max_pixmap_size, 40, 5
-		)
-	);
-}
-
 void
 MainWindow::showNewOpenProjectPanel()
 {
@@ -927,6 +916,10 @@ MainWindow::pageContextMenuRequested(
 void
 MainWindow::pastLastPageContextMenuRequested(QPoint const& screen_pos)
 {
+	if (!isProjectLoaded()) {
+		return;
+	}
+
 	QMenu menu;
 	menu.addAction(QIcon(":/icons/insert-here-16.png"), tr("Insert here ..."));
 	
@@ -1368,6 +1361,11 @@ MainWindow::showAboutDialog()
 	Ui::AboutDialog ui;
 	QDialog* dialog = new QDialog(this);
 	ui.setupUi(dialog);
+	ui.version->setText(QString::fromUtf8(VERSION));
+
+	QResource license(":/GPLv3.html");
+	ui.licenseViewer->setHtml(QString::fromUtf8((char const*)license.data(), license.size()));
+
 	dialog->setAttribute(Qt::WA_DeleteOnClose);
 	dialog->setWindowModality(Qt::WindowModal);
 	dialog->show();
