@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
+	Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "ImageLoader.h"
 #include "AtomicFileOverwriter.h"
 #include "imageproc/Scale.h"
+#include "imageproc/GrayImage.h"
 #include <QCoreApplication>
 #include <QCryptographicHash>
 #include <QThread>
@@ -45,7 +46,7 @@
 
 using namespace ::boost;
 using namespace ::boost::multi_index;
-
+using namespace imageproc;
 
 class ThumbnailPixmapCache::Item
 {
@@ -635,7 +636,7 @@ ThumbnailPixmapCache::Impl::loadSaveThumbnail(
 		return image;
 	}
 	
-	image = ImageLoader::load(image_id.filePath(), image_id.page());
+	image = ImageLoader::load(image_id);
 	if (image.isNull()) {
 		return QImage();
 	}
@@ -668,7 +669,7 @@ ThumbnailPixmapCache::Impl::getThumbFilePath(
 	thumb_file_path += QChar('/');
 	thumb_file_path += orig_img_path.baseName();
 	thumb_file_path += QChar('_');
-	thumb_file_path += QString::number(image_id.page());
+	thumb_file_path += QString::number(image_id.zeroBasedPage());
 	thumb_file_path += QChar('_');
 	thumb_file_path += orig_path_hash_str;
 	thumb_file_path += QString::fromAscii(".png");
@@ -690,7 +691,7 @@ ThumbnailPixmapCache::Impl::makeThumbnail(
 	
 	if (image.format() == QImage::Format_Indexed8 && image.isGrayscale()) {
 		// This will be faster than QImage::scale().
-		return imageproc::scaleToGray(image, to_size);
+		return scaleToGray(GrayImage(image), to_size);
 	}
 	
 	return image.scaled(
