@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,10 +16,12 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "ThumbnailSequence.h"
 #include "ThumbnailSequence.h.moc"
 #include "ThumbnailFactory.h"
 #include "IncompleteThumbnail.h"
 #include "PageSequence.h"
+#include "PageOrderProvider.h"
 #include "PageInfo.h"
 #include "PageId.h"
 #include "ImageId.h"
@@ -30,6 +32,9 @@
 #include <boost/multi_index/ordered_index.hpp>
 #include <boost/multi_index/sequenced_index.hpp>
 #include <boost/multi_index/mem_fun.hpp>
+#include <boost/function.hpp>
+#include <boost/lambda/lambda.hpp>
+#include <boost/lambda/bind.hpp>
 #include <boost/foreach.hpp>
 #include <QGraphicsScene>
 #include <QGraphicsItem>
@@ -65,16 +70,14 @@
 #include <stddef.h>
 #include <assert.h>
 
-using namespace ::boost;
 using namespace ::boost::multi_index;
+using namespace ::boost::lambda;
 
 
 class ThumbnailSequence::Item
 {
 public:
-	Item(PageInfo const& page_info, int page_num, CompositeItem* comp_item)
-	: pageInfo(page_info), pageNum(page_num), composite(comp_item),
-	m_isSelected(false), m_isSelectionLeader(false) {}
+	Item(PageInfo const& page_info, CompositeItem* comp_item);
 	
 	PageId const& pageId() const { return pageInfo.id(); }
 	
@@ -87,14 +90,35 @@ public:
 	void setSelectionLeader(bool selection_leader) const;
 	
 	PageInfo pageInfo;
-	int pageNum;
 	mutable CompositeItem* composite;
+	mutable bool incompleteThumbnail;
 private:
 	mutable bool m_isSelected;
 	mutable bool m_isSelectionLeader;
 };
 
 
+class ThumbnailSequence::GraphicsScene : public QGraphicsScene
+{
+public:
+	typedef boost::function<void (QGraphicsSceneContextMenuEvent*)> ContextMenuEventCallback;
+
+	void setContextMenuEventCallback(ContextMenuEventCallback callback) {
+		m_contextMenuEventCallback = callback;
+	}
+protected:
+	virtual void contextMenuEvent(QGraphicsSceneContextMenuEvent* event) {
+		QGraphicsScene::contextMenuEvent(event);
+		
+		if (!event->isAccepted() && m_contextMenuEventCallback) {
+			m_contextMenuEventCallback(event);
+		}
+	}
+private:
+	ContextMenuEventCallback m_contextMenuEventCallback;
+};
+
+
 class ThumbnailSequence::Impl
 {
 public:
@@ -103,18 +127,33 @@ public:
 	~Impl();
 	
 	void setThumbnailFactory(IntrusivePtr<ThumbnailFactory> const& factory);
-	
+
 	void attachView(QGraphicsView* view);
 	
-	void reset(PageSequenceSnapshot const& pages,
-		SelectionAction const selection_action);
+	void reset(PageSequence const& pages,
+		SelectionAction const selection_action,
+		IntrusivePtr<PageOrderProvider const> const& provider);
+
+	PageSequence toPageSequence() const;
 	
 	void invalidateThumbnail(PageId const& page_id);
+
+	void invalidateThumbnail(PageInfo const& page_info);
 	
 	void invalidateAllThumbnails();
 	
-	void setSelection(PageId const& page_id);
-	
+	bool setSelection(PageId const& page_id);
+
+	PageInfo selectionLeader() const;
+
+	PageInfo prevPage(PageId const& page_id) const;
+
+	PageInfo nextPage(PageId const& page_id) const;
+
+	PageInfo firstPage() const;
+
+	PageInfo lastPage() const;
+
 	void insert(PageInfo const& new_page,
 		BeforeOrAfter before_or_after, ImageId const& image);
 
@@ -151,6 +190,10 @@ private:
 	typedef Container::index<ItemsInOrderTag>::type ItemsInOrder;
 	typedef Container::index<SelectedThenUnselectedTag>::type SelectedThenUnselected;
 	
+	void invalidateThumbnailImpl(ItemsById::iterator id_it);
+
+	void sceneContextMenuEvent(QGraphicsSceneContextMenuEvent* evt);
+
 	void selectItemNoModifiers(ItemsById::iterator const& it);
 	
 	void selectItemWithControl(ItemsById::iterator const& it);
@@ -163,21 +206,35 @@ private:
 	
 	void moveToUnselected(Item const* item);
 	
-	void setThumbnail(
-		ItemsById::iterator const& id_it,
-		std::auto_ptr<CompositeItem> composite);
-	
 	void clear();
 	
 	void clearSelection();
+
+	/**
+	 * Calculates the insertion position for an item with the given PageId
+	 * based on m_ptrOrderProvider.
+	 *
+	 * \param begin Beginning of the interval to consider.
+	 * \param end End of the interval to consider.
+	 * \param page_id The item to find insertion position for.
+	 * \param page_incomplete Whether the page is represented by IncompleteThumbnail.
+	 * \param hint The place to start the search.  Must be within [begin, end].
+	 * \param dist_from_hint If provided, the distance from \p hint
+	 *        to the calculated insertion position will be written there.
+	 *        For example, \p dist_from_hint == -2 would indicate that the
+	 *        insertion position is two elements to the left of \p hint.
+	 */
+	ItemsInOrder::iterator itemInsertPosition(
+		ItemsInOrder::iterator begin, ItemsInOrder::iterator end,
+		PageId const& page_id, bool page_incomplete,
+		ItemsInOrder::iterator hint, int* dist_from_hint = 0);
 	
-	std::auto_ptr<QGraphicsItem> getThumbnail(
-		PageInfo const& page_info, int page_num);
+	std::auto_ptr<QGraphicsItem> getThumbnail(PageInfo const& page_info);
 	
 	std::auto_ptr<LabelGroup> getLabelGroup(PageInfo const& page_info);
 	
 	std::auto_ptr<CompositeItem> getCompositeItem(
-		Item const* item, PageInfo const& info, int page_num);
+		Item const* item, PageInfo const& info);
 	
 	void commitSceneRect();
 	
@@ -196,7 +253,8 @@ private:
 	
 	Item const* m_pSelectionLeader;
 	IntrusivePtr<ThumbnailFactory> m_ptrFactory;
-	QGraphicsScene m_graphicsScene;
+	IntrusivePtr<PageOrderProvider const> m_ptrOrderProvider;
+	GraphicsScene m_graphicsScene;
 	QRectF m_sceneRect;
 };
 
@@ -244,6 +302,8 @@ public:
 	void setItem(Item const* item) { m_pItem = item; }
 	
 	Item const* item() { return m_pItem; }
+
+	bool incompleteThumbnail() const;
 	
 	void updateSceneRect(QRectF& scene_rect);
 	
@@ -272,19 +332,6 @@ private:
 };
 
 
-class ThumbnailSequence::PageIdMutator
-{
-public:
-	PageIdMutator(PageId const& new_page_id) : m_newPageId(new_page_id) {}
-
-	void operator()(Item& item) {
-		item.pageInfo.setId(m_newPageId);
-	}
-private:
-	PageId m_newPageId;
-};
-
-
 /*============================= ThumbnailSequence ===========================*/
 
 ThumbnailSequence::ThumbnailSequence(QSizeF const& max_logical_thumb_size)
@@ -309,10 +356,18 @@ ThumbnailSequence::attachView(QGraphicsView* const view)
 }
 
 void
-ThumbnailSequence::reset(PageSequenceSnapshot const& pages,
-	SelectionAction const selection_action)
+ThumbnailSequence::reset(
+	PageSequence const& pages,
+	SelectionAction const selection_action,
+	IntrusivePtr<PageOrderProvider const> const& order_provider)
 {
-	m_ptrImpl->reset(pages, selection_action);
+	m_ptrImpl->reset(pages, selection_action, order_provider);
+}
+
+PageSequence
+ThumbnailSequence::toPageSequence() const
+{
+	return m_ptrImpl->toPageSequence();
 }
 
 void
@@ -321,16 +376,52 @@ ThumbnailSequence::invalidateThumbnail(PageId const& page_id)
 	m_ptrImpl->invalidateThumbnail(page_id);
 }
 
+void
+ThumbnailSequence::invalidateThumbnail(PageInfo const& page_info)
+{
+	m_ptrImpl->invalidateThumbnail(page_info);
+}
+
 void
 ThumbnailSequence::invalidateAllThumbnails()
 {
 	m_ptrImpl->invalidateAllThumbnails();
 }
 
-void
+bool
 ThumbnailSequence::setSelection(PageId const& page_id)
 {
-	m_ptrImpl->setSelection(page_id);
+	return m_ptrImpl->setSelection(page_id);
+}
+
+PageInfo
+ThumbnailSequence::selectionLeader() const
+{
+	return m_ptrImpl->selectionLeader();
+}
+
+PageInfo
+ThumbnailSequence::prevPage(PageId const& reference_page) const
+{
+	return m_ptrImpl->prevPage(reference_page);
+}
+
+PageInfo
+ThumbnailSequence::nextPage(PageId const& reference_page) const
+{
+	return m_ptrImpl->nextPage(reference_page);
+}
+
+PageInfo
+ThumbnailSequence::firstPage() const
+{
+	return m_ptrImpl->firstPage();
+}
+
+PageInfo
+ThumbnailSequence::lastPage() const
+{
+	return m_ptrImpl->lastPage();
 }
 
 void
@@ -376,13 +467,6 @@ ThumbnailSequence::emitNewSelectionLeader(
 	emit newSelectionLeader(page_info, thumb_rect, flags);
 }
 
-void
-ThumbnailSequence::emitContextMenuRequested(
-	PageInfo const& page_info, QPoint const& screen_pos, bool selected)
-{
-	emit contextMenuRequested(page_info, screen_pos, selected);
-}
-
 
 /*======================== ThumbnailSequence::Impl ==========================*/
 
@@ -396,6 +480,9 @@ ThumbnailSequence::Impl::Impl(
 	m_selectedThenUnselected(m_items.get<SelectedThenUnselectedTag>()),
 	m_pSelectionLeader(0)
 {
+	m_graphicsScene.setContextMenuEventCallback(
+		bind(&Impl::sceneContextMenuEvent, this, _1)
+	);
 }
 
 ThumbnailSequence::Impl::~Impl()
@@ -416,9 +503,13 @@ ThumbnailSequence::Impl::attachView(QGraphicsView* const view)
 }
 
 void
-ThumbnailSequence::Impl::reset(PageSequenceSnapshot const& pages,
-	SelectionAction const selection_action)
+ThumbnailSequence::Impl::reset(
+	PageSequence const& pages,
+	SelectionAction const selection_action,
+	IntrusivePtr<PageOrderProvider const> const& order_provider)
 {
+	m_ptrOrderProvider = order_provider;
+
 	std::set<PageId> selected;
 	PageInfo selection_leader;
 	
@@ -435,25 +526,17 @@ ThumbnailSequence::Impl::reset(PageSequenceSnapshot const& pages,
 	if (num_pages == 0) {
 		return;
 	}
-	
+
 	Item const* some_selected_item = 0;
-	double offset = 0;
-	
+
 	for (size_t i = 0; i < num_pages; ++i) {
 		PageInfo const& page_info(pages.pageAt(i));
 		
-		std::auto_ptr<CompositeItem> composite(
-			getCompositeItem(0, page_info, i)
-		);
-		composite->setPos(0.0, offset);
-		composite->updateSceneRect(m_sceneRect);
-		
-		offset += composite->boundingRect().height() + SPACING;
-		
-		m_itemsInOrder.push_back(Item(page_info, i, composite.get()));
+		std::auto_ptr<CompositeItem> composite(getCompositeItem(0, page_info));
+		m_itemsInOrder.push_back(Item(page_info, composite.release()));
 		Item const* item = &m_itemsInOrder.back();
-		composite->setItem(item);
-		
+		item->composite->setItem(item);
+
 		if (selected.find(page_info.id()) != selected.end()) {
 			item->setSelected(true);
 			moveToSelected(item);
@@ -462,21 +545,13 @@ ThumbnailSequence::Impl::reset(PageSequenceSnapshot const& pages,
 		if (page_info.id() == selection_leader.id()) {
 			m_pSelectionLeader = item;
 		}
-		
-		m_graphicsScene.addItem(composite.release());
 	}
-	
-	commitSceneRect();
+
+	invalidateAllThumbnails();
 	
 	if (!m_pSelectionLeader) {
 		if (some_selected_item) {
 			m_pSelectionLeader = some_selected_item;
-		} else {
-			ItemsById::iterator id_it(m_itemsById.find(pages.curPage().id()));
-			if (id_it != m_itemsById.end()) {
-				m_pSelectionLeader = &*id_it;
-				moveToSelected(m_pSelectionLeader);
-			}
 		}
 	}
 	
@@ -488,55 +563,182 @@ ThumbnailSequence::Impl::reset(PageSequenceSnapshot const& pages,
 	}
 }
 
+PageSequence
+ThumbnailSequence::Impl::toPageSequence() const
+{
+	PageSequence pages;
+
+	BOOST_FOREACH(Item const& item, m_itemsInOrder) {
+		pages.append(item.pageInfo);
+	}
+
+	return pages;
+}
+
 void
 ThumbnailSequence::Impl::invalidateThumbnail(PageId const& page_id)
 {
 	ItemsById::iterator const id_it(m_itemsById.find(page_id));
 	if (id_it != m_itemsById.end()) {
-		setThumbnail(
-			id_it,
-			getCompositeItem(&*id_it, id_it->pageInfo, id_it->pageNum)
-		);
-		id_it->composite->updateAppearence(id_it->isSelected(), id_it->isSelectionLeader());
+		invalidateThumbnailImpl(id_it);
+	}
+}
+
+void
+ThumbnailSequence::Impl::invalidateThumbnail(PageInfo const& page_info)
+{
+	ItemsById::iterator const id_it(m_itemsById.find(page_info.id()));
+	if (id_it != m_itemsById.end()) {
+		m_itemsById.modify(id_it, bind(&Item::pageInfo, _1) = page_info);
+		invalidateThumbnailImpl(id_it);
+	}
+}
+
+void
+ThumbnailSequence::Impl::invalidateThumbnailImpl(ItemsById::iterator const id_it)
+{
+	std::auto_ptr<CompositeItem> composite(
+		getCompositeItem(&*id_it, id_it->pageInfo)
+	);
+	CompositeItem* const new_composite = composite.get();
+	CompositeItem* const old_composite = id_it->composite;
+	QSizeF const old_size(old_composite->boundingRect().size());
+	QSizeF const new_size(new_composite->boundingRect().size());
+	QPointF const old_pos(new_composite->pos());
+	
+	new_composite->updateAppearence(id_it->isSelected(), id_it->isSelectionLeader());
+	
+	m_graphicsScene.addItem(composite.release());
+	id_it->composite = new_composite;
+	id_it->incompleteThumbnail = new_composite->incompleteThumbnail();
+	delete old_composite;
+	
+	ItemsInOrder::iterator after_old(m_items.project<ItemsInOrderTag>(id_it));
+	// Notice after_old++ below.
+
+	// Move our item to the beginning of m_itemsInOrder, to make it out of range
+	// we are going to pass to itemInsertPosition().
+	m_itemsInOrder.relocate(m_itemsInOrder.begin(), after_old++);
+
+	int dist = 0;
+	ItemsInOrder::iterator const after_new(
+		itemInsertPosition(
+			++m_itemsInOrder.begin(), m_itemsInOrder.end(),
+			id_it->pageInfo.id(), id_it->incompleteThumbnail,
+			after_old, &dist
+		)
+	);
+
+	// Move our item to its intended position.
+	m_itemsInOrder.relocate(after_new, m_itemsInOrder.begin());
+
+
+	// Now let's reposition the items on the scene.
+	
+	ItemsInOrder::iterator ord_it, ord_end;
+
+	// The range of [ord_it, ord_end) is supposed to contain all items
+	// between the old and new positions of our item, with the new
+	// position in range.
+
+	if (dist <= 0) { // New position is before or equals to the old one.
+		ord_it = after_new;
+		--ord_it; // Include new item position in the range.
+		ord_end = after_old;
+	} else { // New position is after the old one.
+		ord_it = after_old;
+		ord_end = after_new;
+	}
+	
+	double offset = 0;
+
+	if (ord_it != m_itemsInOrder.begin()) {
+		ItemsInOrder::iterator prev(ord_it);
+		--prev;
+		offset = prev->composite->pos().y() + prev->composite->boundingRect().height() + SPACING;
+	}
+
+	// Reposition items between the old and the new position of our item,
+	// including the item itself.
+	for (; ord_it != ord_end; ++ord_it) {
+		ord_it->composite->setPos(0.0, offset);
+		offset += ord_it->composite->boundingRect().height() + SPACING;
+	}
+
+	// Reposition the items following both the new and the old position
+	// of the item, if the item size has changed.
+	if (old_size != new_size) {
+		for (; ord_it != m_itemsInOrder.end(); ++ord_it) {
+			ord_it->composite->setPos(0.0, offset);
+			offset += ord_it->composite->boundingRect().height() + SPACING;
+		}
+	}
+	
+	// Update scene rect.
+	m_sceneRect.setTop(m_sceneRect.bottom());
+	m_itemsInOrder.front().composite->updateSceneRect(m_sceneRect);
+	m_sceneRect.setBottom(m_sceneRect.top());
+	m_itemsInOrder.back().composite->updateSceneRect(m_sceneRect);
+	id_it->composite->updateSceneRect(m_sceneRect);
+	commitSceneRect();
+
+	// Possibly emit the newSelectionLeader() signal.
+	if (m_pSelectionLeader == &*id_it) {
+		if (old_size != new_size || old_pos != id_it->composite->pos()) {
+			m_rOwner.emitNewSelectionLeader(
+				id_it->pageInfo, id_it->composite, REDUNDANT_SELECTION
+			);
+		}
 	}
 }
 
 void
 ThumbnailSequence::Impl::invalidateAllThumbnails()
 {
-	m_sceneRect = QRectF(0.0, 0.0, 0.0, 0.0);
-	double offset = 0;
-	
+	// Recreate thumbnails now, whether a thumbnail is incomplete
+	// is taken into account when sorting.
 	ItemsInOrder::iterator ord_it(m_itemsInOrder.begin());
 	ItemsInOrder::iterator const ord_end(m_itemsInOrder.end());
 	for (; ord_it != ord_end; ++ord_it) {
-		std::auto_ptr<CompositeItem> composite(
-			getCompositeItem(&*ord_it, ord_it->pageInfo, ord_it->pageNum)
-		);
-		
 		CompositeItem* const old_composite = ord_it->composite;
-		CompositeItem* const new_composite = composite.get();
-		
-		new_composite->setPos(0.0, offset);
-		new_composite->updateSceneRect(m_sceneRect);
-		new_composite->updateAppearence(ord_it->isSelected(), ord_it->isSelectionLeader());
-		
-		offset += new_composite->boundingRect().height() + SPACING;
+		ord_it->composite = getCompositeItem(&*ord_it, ord_it->pageInfo).release();
+		ord_it->incompleteThumbnail = ord_it->composite->incompleteThumbnail();
 		delete old_composite;
-		
-		ord_it->composite = new_composite;
-		m_graphicsScene.addItem(composite.release());
+	}
+
+	// Sort pages in m_itemsInOrder using m_ptrOrderProvider.
+	if (m_ptrOrderProvider.get()) {
+		m_itemsInOrder.sort(
+			bind(
+				&PageOrderProvider::precedes, m_ptrOrderProvider.get(),
+				bind(&Item::pageId, _1), bind(&Item::incompleteThumbnail, _1),
+				bind(&Item::pageId, _2), bind(&Item::incompleteThumbnail, _2) 
+			)
+		);
+	}
+	
+	m_sceneRect = QRectF(0.0, 0.0, 0.0, 0.0);
+	double offset = 0;
+
+	for (ord_it = m_itemsInOrder.begin(); ord_it != ord_end; ++ord_it) {
+		CompositeItem* composite = ord_it->composite;
+		composite->setPos(0.0, offset);
+		composite->updateSceneRect(m_sceneRect);
+		composite->updateAppearence(ord_it->isSelected(), ord_it->isSelectionLeader());
+
+		offset += composite->boundingRect().height() + SPACING;
+		m_graphicsScene.addItem(composite);
 	}
 	
 	commitSceneRect();
 }
 
-void
+bool
 ThumbnailSequence::Impl::setSelection(PageId const& page_id)
 {
 	ItemsById::iterator const id_it(m_itemsById.find(page_id));
 	if (id_it == m_itemsById.end()) {
-		return;
+		return false;
 	}
 	
 	bool const was_selection_leader = (&*id_it == m_pSelectionLeader);
@@ -573,6 +775,82 @@ ThumbnailSequence::Impl::setSelection(PageId const& page_id)
 	}
 	
 	m_rOwner.emitNewSelectionLeader(id_it->pageInfo, id_it->composite, flags);
+
+	return true;
+}
+
+PageInfo
+ThumbnailSequence::Impl::selectionLeader() const
+{
+	if (m_pSelectionLeader) {
+		return m_pSelectionLeader->pageInfo;
+	} else {
+		return PageInfo();
+	}
+}
+
+PageInfo
+ThumbnailSequence::Impl::prevPage(PageId const& reference_page) const
+{
+	ItemsInOrder::iterator ord_it;
+
+	if (m_pSelectionLeader && m_pSelectionLeader->pageInfo.id() == reference_page) {
+		// Common case optimization.
+		ord_it = m_itemsInOrder.iterator_to(*m_pSelectionLeader);
+	} else {
+		ord_it = m_items.project<ItemsInOrderTag>(m_itemsById.find(reference_page));
+	}
+
+	if (ord_it != m_itemsInOrder.end()) {
+		if (ord_it != m_itemsInOrder.begin()) {
+			--ord_it;
+			return ord_it->pageInfo;
+		}
+	}
+
+	return PageInfo();
+}
+
+PageInfo
+ThumbnailSequence::Impl::nextPage(PageId const& reference_page) const
+{
+	ItemsInOrder::iterator ord_it;
+
+	if (m_pSelectionLeader && m_pSelectionLeader->pageInfo.id() == reference_page) {
+		// Common case optimization.
+		ord_it = m_itemsInOrder.iterator_to(*m_pSelectionLeader);
+	} else {
+		ord_it = m_items.project<ItemsInOrderTag>(m_itemsById.find(reference_page));
+	}
+
+	if (ord_it != m_itemsInOrder.end()) {
+		++ord_it;
+		if (ord_it != m_itemsInOrder.end()) {
+			return ord_it->pageInfo;
+		}
+	}
+
+	return PageInfo();
+}
+
+PageInfo
+ThumbnailSequence::Impl::firstPage() const
+{
+	if (m_items.empty()) {
+		return PageInfo();
+	}
+
+	return m_itemsInOrder.front().pageInfo;
+}
+
+PageInfo
+ThumbnailSequence::Impl::lastPage() const
+{
+	if (m_items.empty()) {
+		return PageInfo();
+	}
+
+	return m_itemsInOrder.back().pageInfo;
 }
 
 void
@@ -580,23 +858,42 @@ ThumbnailSequence::Impl::insert(
 	PageInfo const& page_info,
 	BeforeOrAfter before_or_after, ImageId const& image)
 {
-	ItemsById::iterator id_it(m_itemsById.lower_bound(PageId(image)));
-	if (id_it == m_itemsById.end() || id_it->pageInfo.imageId() != image) {
-		return;
+	ItemsInOrder::iterator ord_it;
+
+	if (before_or_after == BEFORE && image.isNull()) {
+		ord_it = m_itemsInOrder.end();
+	} else {
+		// Note that we have to use lower_bound() rather than find() because
+		// we are not searching for PageId(image) exactly, which implies
+		// PageId::SINGLE_PAGE configuration, but rather we search for
+		// a page with any configuration, as long as it references the same image. 
+		ItemsById::iterator id_it(m_itemsById.lower_bound(PageId(image)));
+		if (id_it == m_itemsById.end() || id_it->pageInfo.imageId() != image) {
+			// Reference page not found.
+			return;
+		}
+		
+		ord_it = m_items.project<ItemsInOrderTag>(id_it);
+		
+		if (before_or_after == AFTER) {
+			++ord_it;
+			if (!m_ptrOrderProvider.get()) { 
+				// Advance past not only the target page, but also its other half, if it follows.
+				while (ord_it != m_itemsInOrder.end() && ord_it->pageInfo.imageId() == image) {
+					++ord_it;
+				}
+			}
+		}
 	}
+
+	// If m_ptrOrderProvider is not set, ord_it won't change.
+	ord_it = itemInsertPosition(
+		m_itemsInOrder.begin(), m_itemsInOrder.end(), page_info.id(),
+		/*page_incomplete=*/true, ord_it
+	);
 	
-	ItemsInOrder::iterator ord_it(m_items.project<ItemsInOrderTag>(id_it));
-	if (before_or_after == AFTER) {
-		++ord_it;
-	}
-	
-	int page_num = 0;
 	double offset = 0.0;
 	if (!m_items.empty()) {
-		// That's the best thing we can do here.
-		// A proper solution would require renaming files.
-		page_num = m_itemsInOrder.rbegin()->pageNum + 1;
-		
 		if (ord_it != m_itemsInOrder.end()) {
 			offset = ord_it->composite->pos().y();
 		} else {
@@ -607,14 +904,14 @@ ThumbnailSequence::Impl::insert(
 		}
 	}
 	std::auto_ptr<CompositeItem> composite(
-		getCompositeItem(0, page_info, page_num)
+		getCompositeItem(0, page_info)
 	);
 	composite->setPos(0.0, offset);
 	composite->updateSceneRect(m_sceneRect);
 	
 	QPointF const pos_delta(0.0, composite->boundingRect().height() + SPACING);
 	
-	Item const item(page_info, page_num, composite.get());
+	Item const item(page_info, composite.get());
 	std::pair<ItemsInOrder::iterator, bool> const ins(
 		m_itemsInOrder.insert(ord_it, item)
 	);
@@ -635,14 +932,6 @@ ThumbnailSequence::Impl::removePages(std::set<PageId> const& to_remove)
 {
 	m_sceneRect = QRectF(0, 0, 0, 0);
 
-	// If we are removing a LEFT_PAGE or a RIGHT_PAGE, we have to change
-	// the type of the remaining half to SINGLE_PAGE, as that's what
-	// effectively happens in PageSequence::removePages().
-	// We can't do it in place, as that will prevent the other half
-	// to be remove if it's also in the list.
-	std::vector<ImageId> images_to_singularize;
-	images_to_singularize.reserve(m_items.size());
-
 	std::set<PageId>::const_iterator const to_remove_end(to_remove.end());
 	QPointF pos_delta(0, 0);
 
@@ -661,29 +950,12 @@ ThumbnailSequence::Impl::removePages(std::set<PageId> const& to_remove)
 			if (m_pSelectionLeader == &*ord_it) {
 				m_pSelectionLeader = 0;
 			}
-
-			switch (ord_it->pageInfo.id().subPage()) {
-				case PageId::LEFT_PAGE:
-				case PageId::RIGHT_PAGE:
-					images_to_singularize.push_back(ord_it->pageInfo.imageId());
-					break;
-				default:;
-			}
-
 			pos_delta.ry() -= ord_it->composite->boundingRect().height() + SPACING;
 			delete ord_it->composite;
 			m_itemsInOrder.erase(ord_it++);
 		}
 	}
 
-	ItemsById::iterator const id_end(m_itemsById.end());
-	BOOST_FOREACH(ImageId const& image_id, images_to_singularize) {
-		ItemsById::iterator id_it(m_itemsById.lower_bound(PageId(image_id)));
-		for (; id_it != id_end && id_it->pageInfo.imageId() == image_id; ++id_it) {
-			m_itemsById.modify(id_it, PageIdMutator(PageId(image_id, PageId::SINGLE_PAGE)));
-		}
-	}
-
 	commitSceneRect();
 }
 
@@ -750,9 +1022,8 @@ ThumbnailSequence::Impl::selectedRanges() const
 	
 	ItemsInOrder::iterator it(m_itemsInOrder.begin());
 	ItemsInOrder::iterator const end(m_itemsInOrder.end());
-	int page_idx = 0;
 	for (;;) {
-		for (; it != end && !it->isSelected(); ++it, ++page_idx) {
+		for (; it != end && !it->isSelected(); ++it) {
 			// Skip unselected items.
 		}
 		if (it == end) {
@@ -761,8 +1032,7 @@ ThumbnailSequence::Impl::selectedRanges() const
 		
 		ranges.push_back(PageRange());
 		PageRange& range = ranges.back();
-		range.firstPageIdx = page_idx;
-		for (; it != end && it->isSelected(); ++it, ++page_idx) {
+		for (; it != end && it->isSelected(); ++it) {
 			range.pages.push_back(it->pageInfo.id());
 		}
 	}
@@ -774,7 +1044,23 @@ void
 ThumbnailSequence::Impl::contextMenuRequested(
 	PageInfo const& page_info, QPoint const& screen_pos, bool selected)
 {
-	m_rOwner.emitContextMenuRequested(page_info, screen_pos, selected);
+	emit m_rOwner.pageContextMenuRequested(page_info, screen_pos, selected);
+}
+
+void
+ThumbnailSequence::Impl::sceneContextMenuEvent(QGraphicsSceneContextMenuEvent* evt)
+{
+	if (!m_itemsInOrder.empty()) {
+		CompositeItem* composite = m_itemsInOrder.back().composite;
+		QRectF const last_thumb_rect(
+			composite->mapToScene(composite->boundingRect()).boundingRect()
+		);
+		if (evt->scenePos().y() <= last_thumb_rect.bottom()) {
+			return;
+		}
+	}
+
+	emit m_rOwner.pastLastPageContextMenuRequested(evt->screenPos());
 }
 
 void
@@ -943,37 +1229,6 @@ ThumbnailSequence::Impl::selectItemNoModifiers(ItemsById::iterator const& id_it)
 	m_rOwner.emitNewSelectionLeader(id_it->pageInfo, id_it->composite, flags);
 }
 
-void
-ThumbnailSequence::Impl::setThumbnail(
-	ItemsById::iterator const& id_it, std::auto_ptr<CompositeItem> composite)
-{
-	CompositeItem* const old_composite = id_it->composite;
-	CompositeItem* const new_composite = composite.get();
-	QSizeF const old_size(old_composite->boundingRect().size());
-	QSizeF const new_size(new_composite->boundingRect().size());
-	
-	new_composite->setPos(old_composite->pos());
-	composite->updateSceneRect(m_sceneRect);
-	
-	QPointF const pos_delta(0.0, new_size.height() - old_size.height());
-	delete old_composite;
-	
-	if (pos_delta != QPointF(0.0, 0.0)) {
-		ItemsInOrder::iterator ord_it(m_items.project<ItemsInOrderTag>(id_it));
-		ItemsInOrder::iterator const ord_end(m_itemsInOrder.end());
-		++ord_it; // Skip the item itself.
-		for (; ord_it != ord_end; ++ord_it) {
-			ord_it->composite->setPos(ord_it->composite->pos() + pos_delta);
-			ord_it->composite->updateSceneRect(m_sceneRect);
-		}
-	}
-	
-	id_it->composite = new_composite;
-	m_graphicsScene.addItem(composite.release());
-	
-	commitSceneRect();
-}
-
 void
 ThumbnailSequence::Impl::clear()
 {
@@ -1005,14 +1260,70 @@ ThumbnailSequence::Impl::clearSelection()
 	}
 }
 
+ThumbnailSequence::Impl::ItemsInOrder::iterator
+ThumbnailSequence::Impl::itemInsertPosition(
+	ItemsInOrder::iterator const begin, ItemsInOrder::iterator const end,
+	PageId const& page_id, bool const page_incomplete,
+	ItemsInOrder::iterator const hint, int* dist_from_hint)
+{
+	// Note that to preserve stable ordering, this function *must* return hint,
+	// as long as it's an acceptable position.
+
+	if (!m_ptrOrderProvider.get()) {
+		if (dist_from_hint) {
+			*dist_from_hint = 0;
+		}
+		return hint;
+	}
+
+	ItemsInOrder::iterator ins_pos(hint);
+	int dist = 0;
+
+	// While the element immediately preceeding ins_pos is supposed to
+	// follow the page we are inserting, move ins_pos one element back.
+	while (ins_pos != begin) {
+		ItemsInOrder::iterator prev(ins_pos);
+		--prev;
+		bool const precedes = m_ptrOrderProvider->precedes(
+			page_id, page_incomplete, prev->pageId(), prev->incompleteThumbnail
+		);
+		if (precedes) {
+			ins_pos = prev;
+			--dist;
+		} else {
+			break;
+		}
+	}
+
+	// While the element pointed to by ins_pos is supposed to precede
+	// the page we are inserting, advance ins_pos.
+	while (ins_pos != end) {
+		bool const precedes = m_ptrOrderProvider->precedes(
+			ins_pos->pageId(), ins_pos->incompleteThumbnail,
+			page_id, page_incomplete
+		);
+		if (precedes) {
+			++ins_pos;
+			++dist;
+		} else {
+			break;
+		}
+	}
+
+	if (dist_from_hint) {
+		*dist_from_hint = dist;
+	}
+
+	return ins_pos;
+}
+
 std::auto_ptr<QGraphicsItem>
-ThumbnailSequence::Impl::getThumbnail(
-	PageInfo const& page_info, int const page_num)
+ThumbnailSequence::Impl::getThumbnail(PageInfo const& page_info)
 {
 	std::auto_ptr<QGraphicsItem> thumb;
 	
 	if (m_ptrFactory.get()) {
-		thumb = m_ptrFactory->get(page_info, page_num);
+		thumb = m_ptrFactory->get(page_info);
 	}
 	
 	if (!thumb.get()) {
@@ -1028,13 +1339,12 @@ ThumbnailSequence::Impl::getLabelGroup(PageInfo const& page_info)
 	PageId const& page_id = page_info.id();
 	QFileInfo const file_info(page_id.imageId().filePath());
 	QString const file_name(file_info.fileName());
-	int const page_num = page_id.imageId().page();
 	
 	QString text(file_name);
-	if (page_info.isMultiPageFile() || page_num > 0) {
+	if (page_info.imageId().isMultiPageFile()) {
 		text = ThumbnailSequence::tr(
 			"%1 (page %2)"
-		).arg(file_name).arg(page_num + 1);
+		).arg(file_name).arg(page_id.imageId().page());
 	}
 	
 	std::auto_ptr<QGraphicsSimpleTextItem> normal_text_item(new QGraphicsSimpleTextItem);
@@ -1082,9 +1392,9 @@ ThumbnailSequence::Impl::getLabelGroup(PageInfo const& page_info)
 
 std::auto_ptr<ThumbnailSequence::CompositeItem>
 ThumbnailSequence::Impl::getCompositeItem(
-	Item const* item, PageInfo const& page_info, int const page_num)
+	Item const* item, PageInfo const& page_info)
 {
-	std::auto_ptr<QGraphicsItem> thumb(getThumbnail(page_info, page_num));
+	std::auto_ptr<QGraphicsItem> thumb(getThumbnail(page_info));
 	std::auto_ptr<LabelGroup> label_group(getLabelGroup(page_info));
 	std::auto_ptr<CompositeItem> composite(
 		new CompositeItem(*this, thumb, label_group)
@@ -1106,6 +1416,15 @@ ThumbnailSequence::Impl::commitSceneRect()
 
 /*==================== ThumbnailSequence::Item ======================*/
 
+ThumbnailSequence::Item::Item(PageInfo const& page_info, CompositeItem* comp_item)
+:	pageInfo(page_info),
+	composite(comp_item),
+	incompleteThumbnail(comp_item->incompleteThumbnail()),
+	m_isSelected(false),
+	m_isSelectionLeader(false)
+{
+}
+
 void
 ThumbnailSequence::Item::setSelected(bool selected) const
 {
@@ -1225,6 +1544,12 @@ ThumbnailSequence::CompositeItem::CompositeItem(
 	setZValue(-1);
 }
 
+bool
+ThumbnailSequence::CompositeItem::incompleteThumbnail() const
+{
+	return dynamic_cast<IncompleteThumbnail*>(m_pThumb) != 0;
+}
+
 void
 ThumbnailSequence::CompositeItem::updateSceneRect(QRectF& scene_rect)
 {
