@@ -32,14 +32,17 @@
 #include <QCoreApplication>
 #include <QDomDocument>
 #include <QDomElement>
+#include "CommandLine.h"
 
 namespace deskew
 {
 
-Filter::Filter()
+Filter::Filter(PageSelectionAccessor const& page_selection_accessor)
 :	m_ptrSettings(new Settings)
 {
-	m_ptrOptionsWidget.reset(new OptionsWidget(m_ptrSettings));
+	if (CommandLine::get().isGui()) {
+		m_ptrOptionsWidget.reset(new OptionsWidget(m_ptrSettings, page_selection_accessor));
+	}
 }
 
 Filter::~Filter()
