@@ -28,6 +28,7 @@
 
 class PageId;
 class QString;
+class PageSelectionAccessor;
 
 namespace select_content
 {
@@ -47,7 +48,7 @@ class Filter : public AbstractFilter
 {
 	DECLARE_NON_COPYABLE(Filter)
 public:
-	Filter();
+	Filter(PageSelectionAccessor const& page_selection_accessor);
 	
 	virtual ~Filter();
 	
@@ -72,6 +73,7 @@ public:
 		IntrusivePtr<select_content::CacheDrivenTask> const& next_task);
 	
 	OptionsWidget* optionsWidget() { return m_ptrOptionsWidget.get(); }
+	Settings* getSettings() { return m_ptrSettings.get(); };
 private:
 	void writePageSettings(
 		QDomDocument& doc, QDomElement& filter_el,
