@@ -19,6 +19,7 @@
 #include "Settings.h"
 #include "Utils.h"
 #include <QMutexLocker>
+#include <boost/foreach.hpp>
 
 namespace deskew
 {
@@ -65,4 +66,13 @@ Settings::getPageParams(PageId const& page_id) const
 	}
 }
 
+void
+Settings::setDegress(std::set<PageId> const& pages, Params const& params)
+{
+	QMutexLocker const locker(&m_mutex);
+	BOOST_FOREACH(PageId const& page, pages) {
+		Utils::mapSetValue(m_perPageParams, page, params);
+	}
+}
+
 } // namespace deskew
