@@ -26,12 +26,19 @@
 #include "Despeckle.h"
 #include "Undistort.h"
 #include "RenderParams.h"
+#include "dewarping/DistortionModel.h"
 #include "Dpi.h"
 #include "Dpm.h"
 #include "Zone.h"
 #include "ZoneSet.h"
 #include "PictureLayerProperty.h"
 #include "FillColorProperty.h"
+#include "dewarping/CylindricalSurfaceDewarper.h"
+#include "dewarping/TextLineTracer.h"
+#include "dewarping/TopBottomEdgeTracer.h"
+#include "dewarping/DistortionModelBuilder.h"
+#include "dewarping/DewarpingPointMapper.h"
+#include "dewarping/RasterDewarper.h"
 #include "imageproc/GrayImage.h"
 #include "imageproc/BinaryImage.h"
 #include "imageproc/BinaryThreshold.h"
@@ -52,8 +59,12 @@
 #include "imageproc/DrawOver.h"
 #include "imageproc/AdjustBrightness.h"
 #include "imageproc/PolygonRasterizer.h"
+#include "imageproc/ConnectivityMap.h"
+#include "imageproc/InfluenceMap.h"
 #include "config.h"
 #include <boost/foreach.hpp>
+#include <boost/bind.hpp>
+#include <boost/shared_ptr.hpp>
 #include <QImage>
 #include <QSize>
 #include <QPoint>
@@ -69,12 +80,14 @@
 #include <QDebug>
 #include <Qt>
 #include <vector>
+#include <memory>
 #include <algorithm>
 #include <assert.h>
 #include <string.h>
 #include <stdint.h>
 
 using namespace imageproc;
+using namespace dewarping;
 
 namespace output
 {
@@ -156,6 +169,21 @@ void reserveBlackAndWhite(QSize size, int stride, PixelType* data)
 	}
 }
 
+void reserveBlackAndWhite(QImage& img)
+{
+	assert(img.depth() == 8 || img.depth() == 24 || img.depth() == 32);
+	switch (img.format()) {
+		case QImage::Format_Indexed8:
+			reserveBlackAndWhite(img.size(), img.bytesPerLine(), img.bits());
+			break;
+		case QImage::Format_RGB32:
+		case QImage::Format_ARGB32:
+			reserveBlackAndWhite(img.size(), img.bytesPerLine()/4, (uint32_t*)img.bits());
+			break;
+		default:; // Should not happen.
+	}
+}
+
 /**
  * Fills areas of \p mixed with pixels from \p bw_content in
  * areas where \p bw_mask is black.  Supported \p mixed image formats
@@ -206,56 +234,32 @@ void combineMixed(
 
 } // anonymous namespace
 
+
 OutputGenerator::OutputGenerator(
 	Dpi const& dpi, ColorParams const& color_params,
 	DespeckleLevel const despeckle_level,
-	ImageTransformation const& pre_xform,
-	QPolygonF const& content_rect_phys,
-	QPolygonF const& page_rect_phys)
+	ImageTransformation const& xform,
+	QPolygonF const& content_rect_phys)
 :	m_dpi(dpi),
 	m_colorParams(color_params),
-	m_pageRectPhys(page_rect_phys),
-	m_toUncropped(pre_xform.transform()),
+	m_xform(xform),
+	m_outRect(xform.resultingRect().toRect()),
+	m_contentRect(xform.transform().map(content_rect_phys).boundingRect().toRect()),
 	m_despeckleLevel(despeckle_level)
-{
-	QTransform const post_scale(
-		Utils::scaleFromToDpi(pre_xform.preScaledDpi(), m_dpi)
-	);
-	m_toUncropped *= post_scale;
-	
-	m_contentRect = m_toUncropped.map(
-		content_rect_phys
-	).boundingRect().toRect();
-	
-	m_cropRect = m_toUncropped.map(
-		page_rect_phys
-	).boundingRect().toRect();
-	
-	m_fullPageRect = post_scale.map(
-		pre_xform.resultingCropArea()
-	).boundingRect().toRect();
-	
-	// Make sure both m_cropRect and m_fullPageRect completely
-	// cover m_contentRect.
-	m_cropRect |= m_contentRect;
-	m_fullPageRect |= m_contentRect;
-}
+{	
+	assert(m_outRect.topLeft() == QPoint(0, 0));
 
-QTransform
-OutputGenerator::toOutput() const
-{
-	QTransform xform(m_toUncropped);
-
-	QPointF const delta(-m_cropRect.topLeft());
-	xform *= QTransform().translate(delta.x(), delta.y());
-
-	return xform;
+	// Note that QRect::contains(<empty rect>) always returns false, so we don't use it here.
+	assert(m_outRect.contains(m_contentRect.topLeft()) && m_outRect.contains(m_contentRect.bottomRight()));
 }
 
 QImage
 OutputGenerator::process(
 	TaskStatus const& status, FilterData const& input,
 	ZoneSet const& picture_zones, ZoneSet const& fill_zones,
+	DewarpingMode dewarping_mode,
+	DistortionModel& distortion_model,
+	DepthPerception const& depth_perception,
 	imageproc::BinaryImage* auto_picture_mask,
 	imageproc::BinaryImage* speckles_image,
 	DebugImages* const dbg) const
@@ -263,6 +267,7 @@ OutputGenerator::process(
 	QImage image(
 		processImpl(
 			status, input, picture_zones, fill_zones,
+			dewarping_mode, distortion_model, depth_perception,
 			auto_picture_mask, speckles_image, dbg
 		)
 	);
@@ -279,75 +284,25 @@ OutputGenerator::process(
 QSize
 OutputGenerator::outputImageSize() const
 {
-	return m_cropRect.size();
+	return m_outRect.size();
 }
 
 QRect
 OutputGenerator::outputContentRect() const
 {
-	return QRect(m_contentRect.topLeft() - m_cropRect.topLeft(), m_contentRect.size());
+	return m_contentRect;
 }
 
-QImage
-OutputGenerator::processAsIs(FilterData const& input,
-	TaskStatus const& status, DebugImages* const dbg) const
-{
-	uint8_t const dominant_gray = reserveBlackAndWhite<uint8_t>(
-		calcDominantBackgroundGrayLevel(input.grayImage())
-	);
-	
-	status.throwIfCancelled();
-	
-	QColor const bg_color(dominant_gray, dominant_gray, dominant_gray);
-	
-	QImage out;
-
-	if (input.origImage().allGray()) {
-		if (m_cropRect.isEmpty()) {
-			QImage image(1, 1, QImage::Format_Indexed8);
-			image.setColorTable(createGrayscalePalette());
-			image.fill(dominant_gray);
-			return image;
-		}
-
-		out = transformToGray(
-			input.grayImage(), m_toUncropped, m_cropRect, bg_color
-		);
-	} else {
-		if (m_cropRect.isEmpty()) {
-			QImage image(1, 1, QImage::Format_RGB32);
-			image.fill(bg_color.rgb());
-			return image;
-		}
-
-		out = transform(input.origImage(), m_toUncropped, m_cropRect, bg_color);
-	}
-
-	assert(out.depth() == 8 || out.depth() == 24 || out.depth() == 32);
-	switch (out.format()) {
-		case QImage::Format_Indexed8:
-			reserveBlackAndWhite(out.size(), out.bytesPerLine(), out.bits());
-			break;
-		case QImage::Format_RGB32:
-		case QImage::Format_ARGB32:
-			reserveBlackAndWhite(out.size(), out.bytesPerLine()/4, (uint32_t*)out.bits());
-			break;
-		default:; // Should not happen.
-	}
-
-	return out;
-}
-
-QImage
+GrayImage
 OutputGenerator::normalizeIlluminationGray(
 	TaskStatus const& status,
 	QImage const& input, QPolygonF const& area_to_consider,
-	QTransform const& xform, QRect const& target_rect, DebugImages* const dbg)
+	QTransform const& xform, QRect const& target_rect,
+	GrayImage* background, DebugImages* const dbg)
 {
 	GrayImage to_be_normalized(
 		transformToGray(
-			input, xform, target_rect,
-			Qt::black // <-- Important!
+			input, xform, target_rect, OutsidePixels::assumeWeakNearest()
 		)
 	);
 	if (dbg) {
@@ -368,19 +323,22 @@ OutputGenerator::normalizeIlluminationGray(
 	
 	status.throwIfCancelled();
 	
-	GrayImage background(bg_ps.render(to_be_normalized.size()));
+	GrayImage bg_img(bg_ps.render(to_be_normalized.size()));
 	if (dbg) {
-		dbg->add(background, "background");
+		dbg->add(bg_img, "background");
+	}
+	if (background) {
+		*background = bg_img;
 	}
 	
 	status.throwIfCancelled();
 	
-	grayRasterOp<RaiseAboveBackground>(background, to_be_normalized);
+	grayRasterOp<RaiseAboveBackground>(bg_img, to_be_normalized);
 	if (dbg) {
-		dbg->add(background, "normalized_illumination");
+		dbg->add(bg_img, "normalized_illumination");
 	}
 	
-	return background;
+	return bg_img;
 }
 
 imageproc::BinaryImage
@@ -454,7 +412,8 @@ OutputGenerator::modifyBinarizationMask(
 	imageproc::BinaryImage& bw_mask,
 	QRect const& mask_rect, ZoneSet const& zones) const
 {
-	QTransform xform(m_toUncropped * QTransform().translate(-mask_rect.x(), -mask_rect.y()));
+	QTransform xform(m_xform.transform());
+	xform *= QTransform().translate(-mask_rect.x(), -mask_rect.y());
 
 	typedef PictureLayerProperty PLP;
 
@@ -487,26 +446,102 @@ QImage
 OutputGenerator::processImpl(
 	TaskStatus const& status, FilterData const& input,
 	ZoneSet const& picture_zones, ZoneSet const& fill_zones,
+	DewarpingMode dewarping_mode,
+	DistortionModel& distortion_model,
+	DepthPerception const& depth_perception,
 	imageproc::BinaryImage* auto_picture_mask,
 	imageproc::BinaryImage* speckles_image,
 	DebugImages* const dbg) const
 {
 	RenderParams const render_params(m_colorParams);
-	
-	if (!render_params.whiteMargins()) {
-		QImage out(processAsIs(input, status, dbg));
-		applyFillZonesInPlace(out, fill_zones);
-		return out;
+
+	if (dewarping_mode == DewarpingMode::AUTO ||
+		(dewarping_mode == DewarpingMode::MANUAL && distortion_model.isValid())) {
+		return processWithDewarping(
+			status, input, picture_zones, fill_zones,
+			dewarping_mode, distortion_model, depth_perception,
+			auto_picture_mask, speckles_image, dbg
+		);
+	} else if (!render_params.whiteMargins()) {
+		return processAsIs(
+			input, status, fill_zones, depth_perception, dbg
+		);
+	} else {
+		return processWithoutDewarping(
+			status, input, picture_zones, fill_zones,
+			auto_picture_mask, speckles_image, dbg
+		);
 	}
+}
+
+QImage
+OutputGenerator::processAsIs(
+	FilterData const& input, TaskStatus const& status,
+	ZoneSet const& fill_zones,
+	DepthPerception const& depth_perception,
+	DebugImages* const dbg) const
+{
+	uint8_t const dominant_gray = reserveBlackAndWhite<uint8_t>(
+		calcDominantBackgroundGrayLevel(input.grayImage())
+	);
+	
+	status.throwIfCancelled();
+	
+	QColor const bg_color(dominant_gray, dominant_gray, dominant_gray);
+	
+	QImage out;
+
+	if (input.origImage().allGray()) {
+		if (m_outRect.isEmpty()) {
+			QImage image(1, 1, QImage::Format_Indexed8);
+			image.setColorTable(createGrayscalePalette());
+			image.fill(dominant_gray);
+			return image;
+		}
+
+		out = transformToGray(
+			input.grayImage(), m_xform.transform(), m_outRect,
+			OutsidePixels::assumeColor(bg_color)
+		);
+	} else {
+		if (m_outRect.isEmpty()) {
+			QImage image(1, 1, QImage::Format_RGB32);
+			image.fill(bg_color.rgb());
+			return image;
+		}
+		
+		out = transform(
+			input.origImage(), m_xform.transform(), m_outRect,
+			OutsidePixels::assumeColor(bg_color)
+		);
+	}
+
+	applyFillZonesInPlace(out, fill_zones);
+	reserveBlackAndWhite(out);
+
+	return out;
+}
+
+QImage
+OutputGenerator::processWithoutDewarping(
+	TaskStatus const& status, FilterData const& input,
+	ZoneSet const& picture_zones, ZoneSet const& fill_zones,
+	imageproc::BinaryImage* auto_picture_mask,
+	imageproc::BinaryImage* speckles_image,
+	DebugImages* dbg) const
+{
+	RenderParams const render_params(m_colorParams);
 	
 	// The whole image minus the part cut off by the split line.
-	QRect const big_margins_rect(m_fullPageRect);
+	QRect const big_margins_rect(
+		m_xform.resultingPreCropArea().boundingRect().toRect() | m_contentRect
+	);
 	
 	// For various reasons, we need some whitespace around the content
 	// area.  This is the number of pixels of such whitespace.
 	int const content_margin = m_dpi.vertical() * 20 / 300;
 	
-	// The content area (in m_toUncropped coordinates) extended
+	// The content area (in output image coordinates) extended
 	// with content_margin.  Note that we prevent that extension
 	// from reaching the neighboring page.
 	QRect const small_margins_rect(
@@ -533,26 +568,24 @@ OutputGenerator::processImpl(
 	
 	// Crop area in original image coordinates.
 	QPolygonF const orig_image_crop_area(
-		input.xform().transformBack().map(
-			input.xform().resultingCropArea()
+		m_xform.transformBack().map(
+			m_xform.resultingPreCropArea()
 		)
 	);
 	
-	// Crop area in maybe_normalized image coordinates.
-	QPolygonF normalize_illumination_crop_area(
-		m_toUncropped.map(orig_image_crop_area)
-	);
+	// Crop area in maybe_normalized image coordinates.	
+	QPolygonF normalize_illumination_crop_area(m_xform.resultingPreCropArea());
 	normalize_illumination_crop_area.translate(-normalize_illumination_rect.topLeft());
-	
+
 	if (render_params.normalizeIllumination()) {
 		maybe_normalized = normalizeIlluminationGray(
 			status, input.grayImage(), orig_image_crop_area,
-			m_toUncropped, normalize_illumination_rect, dbg
+			m_xform.transform(), normalize_illumination_rect, 0, dbg
 		);
 	} else {
 		maybe_normalized = transform(
-			input.origImage(), m_toUncropped,
-			normalize_illumination_rect, Qt::white
+			input.origImage(), m_xform.transform(),
+			normalize_illumination_rect, OutsidePixels::assumeColor(Qt::white)
 		);
 	}
 
@@ -572,8 +605,8 @@ OutputGenerator::processImpl(
 	
 	status.throwIfCancelled();
 	
-	if (render_params.binaryOutput() || m_cropRect.isEmpty()) {
-		BinaryImage dst(m_cropRect.size().expandedTo(QSize(1, 1)), WHITE);
+	if (render_params.binaryOutput() || m_outRect.isEmpty()) {
+		BinaryImage dst(m_outRect.size().expandedTo(QSize(1, 1)), WHITE);
 		
 		if (!m_contentRect.isEmpty()) {
 			BinaryImage bw_content(
@@ -593,27 +626,16 @@ OutputGenerator::processImpl(
 			status.throwIfCancelled();
 			
 			QRect const src_rect(m_contentRect.translated(-normalize_illumination_rect.topLeft()));
-			QRect const dst_rect(m_contentRect.translated(-m_cropRect.topLeft()));
+			QRect const dst_rect(m_contentRect);
 			rasterOp<RopSrc>(dst, dst_rect, bw_content, src_rect.topLeft());
 			bw_content.release(); // Save memory.
-
-#if defined(ENABLE_DEWARPING)
-			if (render_params.dewarp()) {
-				dst = undistort(dst, status, dbg);
-				if (dbg) {
-					dbg->add(dst, "undistorted");
-				}
-			}
-
-			status.throwIfCancelled();
-#endif
 			
 			// It's important to keep despeckling the very last operation
 			// affecting the binary part of the output. That's because
 			// we will be reconstructing the input to this despeckling
 			// operation from the final output file.
 			maybeDespeckleInPlace(
-				dst, m_cropRect, m_cropRect, m_despeckleLevel,
+				dst, m_outRect, m_outRect, m_despeckleLevel,
 				speckles_image, m_dpi, status, dbg
 			);
 		}
@@ -622,7 +644,7 @@ OutputGenerator::processImpl(
 		return dst.toQImage();
 	}
 	
-	QSize const target_size(m_cropRect.size().expandedTo(QSize(1, 1)));
+	QSize const target_size(m_outRect.size().expandedTo(QSize(1, 1)));
 
 	BinaryImage bw_mask;
 	if (render_params.mixedOutput()) {
@@ -647,7 +669,7 @@ OutputGenerator::processImpl(
 
 			if (!m_contentRect.isEmpty()) {
 				QRect const src_rect(m_contentRect.translated(-small_margins_rect.topLeft()));
-				QRect const dst_rect(m_contentRect.translated(-m_cropRect.topLeft()));
+				QRect const dst_rect(m_contentRect);
 				rasterOp<RopSrc>(*auto_picture_mask, dst_rect, bw_mask, src_rect.topLeft());
 			}
 		}
@@ -665,9 +687,9 @@ OutputGenerator::processImpl(
 		assert(maybe_normalized.format() == QImage::Format_Indexed8);
 		QImage tmp(
 			transform(
-				input.origImage(), m_toUncropped,
+				input.origImage(), m_xform.transform(),
 				normalize_illumination_rect,
-				Qt::white
+				OutsidePixels::assumeColor(Qt::white)
 			)
 		);
 		
@@ -680,7 +702,10 @@ OutputGenerator::processImpl(
 		}
 	}
 	
-	if (render_params.mixedOutput()) {
+	if (!render_params.mixedOutput()) {
+		// It's "Color / Grayscale" mode, as we handle B/W above.
+		reserveBlackAndWhite(maybe_normalized);
+	} else {
 		BinaryImage bw_content(
 			binarize(maybe_smoothed, normalize_illumination_crop_area, &bw_mask)
 		);
@@ -734,14 +759,18 @@ OutputGenerator::processImpl(
 	QImage dst(target_size, maybe_normalized.format());
 	if (maybe_normalized.format() == QImage::Format_Indexed8) {
 		dst.setColorTable(createGrayscalePalette());
-		dst.fill(0xff); // White.
+		// White.  0xff is reserved if in "Color / Grayscale" mode.
+		uint8_t const color = render_params.mixedOutput() ? 0xff : 0xfe;
+		dst.fill(color);
 	} else {
-		dst.fill(0xffffffff); // Opaque white.
+		// White.  0x[ff]ffffff is reserved if in "Color / Grayscale" mode.
+		uint32_t const color = render_params.mixedOutput() ? 0xffffffff : 0xfffefefe;
+		dst.fill(color);
 	}
 
 	if (!m_contentRect.isEmpty()) {
 		QRect const src_rect(m_contentRect.translated(-small_margins_rect.topLeft()));
-		QRect const dst_rect(m_contentRect.translated(-m_cropRect.topLeft()));
+		QRect const dst_rect(m_contentRect);
 		drawOver(dst, dst_rect, maybe_normalized, src_rect);
 	}
 	
@@ -749,6 +778,482 @@ OutputGenerator::processImpl(
 	return dst;
 }
 
+QImage
+OutputGenerator::processWithDewarping(
+	TaskStatus const& status, FilterData const& input,
+	ZoneSet const& picture_zones, ZoneSet const& fill_zones,
+	DewarpingMode dewarping_mode,
+	DistortionModel& distortion_model,
+	DepthPerception const& depth_perception,
+	imageproc::BinaryImage* auto_picture_mask,
+	imageproc::BinaryImage* speckles_image,
+	DebugImages* dbg) const
+{
+	QSize const target_size(m_outRect.size().expandedTo(QSize(1, 1)));
+	if (m_outRect.isEmpty()) {
+		return BinaryImage(target_size, WHITE).toQImage();
+	}
+
+	RenderParams const render_params(m_colorParams);
+	
+	// The whole image minus the part cut off by the split line.
+	QRect const big_margins_rect(
+		m_xform.resultingPreCropArea().boundingRect().toRect() | m_contentRect
+	);
+	
+	// For various reasons, we need some whitespace around the content
+	// area.  This is the number of pixels of such whitespace.
+	int const content_margin = m_dpi.vertical() * 20 / 300;
+	
+	// The content area (in output image coordinates) extended
+	// with content_margin.  Note that we prevent that extension
+	// from reaching the neighboring page.
+	QRect const small_margins_rect(
+		m_contentRect.adjusted(
+			-content_margin, -content_margin,
+			content_margin, content_margin
+		).intersected(big_margins_rect)
+	);
+	
+	// This is the area we are going to pass to estimateBackground().
+	// estimateBackground() needs some margins around content, and
+	// generally smaller margins are better, except when there is
+	// some garbage that connects the content to the edge of the
+	// image area.
+	QRect const normalize_illumination_rect(
+#if 1
+		small_margins_rect
+#else
+		big_margins_rect
+#endif
+	);
+	
+	// Crop area in original image coordinates.
+	QPolygonF const orig_image_crop_area(
+		m_xform.transformBack().map(m_xform.resultingPreCropArea())
+	);
+	
+	// Crop area in maybe_normalized image coordinates.
+	QPolygonF normalize_illumination_crop_area(m_xform.resultingPreCropArea());
+	normalize_illumination_crop_area.translate(-normalize_illumination_rect.topLeft());
+	
+	bool const color_original = !input.origImage().allGray();
+
+	// Original image, but:
+	// 1. In a format we can handle, that is grayscale, RGB32, ARGB32
+	// 2. With illumination normalized over the content area, if required.
+	// 3. With margins filled with white, if required.
+	QImage normalized_original;
+
+	// The output we would get if dewarping was turned off, except always grayscale.
+	// Used for automatic picture detection and binarization threshold calculation.
+	// This image corresponds to the area of normalize_illumination_rect above.
+	GrayImage warped_gray_output;
+
+	// Picture mask (white indicate a picture) in the same coordinates as
+	// warped_gray_output.  Only built for Mixed mode.
+	BinaryImage warped_bw_mask;
+
+	BinaryThreshold bw_threshold(128);
+
+	QTransform const norm_illum_to_original(
+		QTransform().translate(
+			normalize_illumination_rect.left(),
+			normalize_illumination_rect.top()
+		) * m_xform.transformBack()
+	);
+
+	if (!render_params.normalizeIllumination()) {
+		if (color_original) {
+			normalized_original = convertToRGBorRGBA(input.origImage());
+		} else {
+			normalized_original = input.grayImage();
+		}
+		if (dewarping_mode == DewarpingMode::AUTO) {
+			warped_gray_output = transformToGray(
+				input.grayImage(), m_xform.transform(), normalize_illumination_rect,
+				OutsidePixels::assumeWeakColor(Qt::white)
+			);
+		} // Otherwise we just don't need it.
+	} else {
+		GrayImage warped_gray_background;
+		warped_gray_output = normalizeIlluminationGray(
+			status, input.grayImage(), orig_image_crop_area,
+			m_xform.transform(), normalize_illumination_rect,
+			&warped_gray_background, dbg
+		);
+
+		status.throwIfCancelled();
+		
+		// Transform warped_gray_background to original image coordinates.
+		warped_gray_background = transformToGray(
+			warped_gray_background.toQImage(), norm_illum_to_original,
+			input.origImage().rect(), OutsidePixels::assumeWeakColor(Qt::black)
+		);
+		if (dbg) {
+			dbg->add(warped_gray_background, "orig_background");
+		}
+
+		status.throwIfCancelled();
+
+		// Turn background into a grayscale, illumination-normalized image.
+		grayRasterOp<RaiseAboveBackground>(warped_gray_background, input.grayImage());
+		if (dbg) {
+			dbg->add(warped_gray_background, "norm_illum_gray");
+		}
+
+		status.throwIfCancelled();
+
+		if (!color_original || render_params.binaryOutput()) {
+			normalized_original = warped_gray_background;
+		} else {
+			normalized_original = convertToRGBorRGBA(input.origImage());
+			adjustBrightnessGrayscale(normalized_original, warped_gray_background);
+			if (dbg) {
+				dbg->add(normalized_original, "norm_illum_color");
+			}
+		}
+	}
+
+	status.throwIfCancelled();
+
+	if (render_params.binaryOutput()) {
+		bw_threshold = calcBinarizationThreshold(
+			warped_gray_output, normalize_illumination_crop_area
+		);
+
+		status.throwIfCancelled();
+
+	} else if (render_params.mixedOutput()) {
+
+		estimateBinarizationMask(
+			status, GrayImage(warped_gray_output),
+			normalize_illumination_rect,
+			small_margins_rect, dbg
+		).swap(warped_bw_mask);
+		if (dbg) {
+			dbg->add(warped_bw_mask, "warped_bw_mask");
+		}
+
+		status.throwIfCancelled();
+
+		if (auto_picture_mask) {
+			if (auto_picture_mask->size() != target_size) {
+				BinaryImage(target_size).swap(*auto_picture_mask);
+			}
+			auto_picture_mask->fill(BLACK);
+
+			if (!m_contentRect.isEmpty()) {
+				QRect const src_rect(m_contentRect.translated(-small_margins_rect.topLeft()));
+				QRect const dst_rect(m_contentRect);
+				rasterOp<RopSrc>(*auto_picture_mask, dst_rect, warped_bw_mask, src_rect.topLeft());
+			}
+		}
+
+		status.throwIfCancelled();
+
+		modifyBinarizationMask(warped_bw_mask, small_margins_rect, picture_zones);
+		if (dbg) {
+			dbg->add(warped_bw_mask, "warped_bw_mask with zones");
+		}
+
+		status.throwIfCancelled();
+
+		// For Mixed output, we mask out pictures when calculating binarization threshold.
+		bw_threshold = calcBinarizationThreshold(
+			warped_gray_output, normalize_illumination_crop_area, &warped_bw_mask
+		);
+		
+		status.throwIfCancelled();
+	}
+
+	if (dewarping_mode == DewarpingMode::AUTO) {
+		DistortionModelBuilder model_builder(Vec2d(0, 1));
+
+		QRect const content_rect(
+			m_contentRect.translated(-normalize_illumination_rect.topLeft())
+		);
+		TextLineTracer::trace(
+			warped_gray_output, m_dpi, content_rect, model_builder, status, dbg
+		);
+		model_builder.transform(norm_illum_to_original);
+
+		TopBottomEdgeTracer::trace(
+			input.grayImage(), model_builder.verticalBounds(),
+			model_builder, status, dbg
+		);
+		
+		distortion_model = model_builder.tryBuildModel(dbg, &input.grayImage().toQImage());
+		if (!distortion_model.isValid()) {
+			setupTrivialDistortionModel(distortion_model);
+		}
+	}
+
+	warped_gray_output = GrayImage(); // Save memory.
+
+	if (render_params.whiteMargins()) {
+		// Fill everything except the content area in normalized_original to white.
+		QPolygonF const orig_content_poly(m_xform.transformBack().map(QRectF(m_contentRect)));
+		fillMarginsInPlace(normalized_original, orig_content_poly, Qt::white);
+		if (dbg) {
+			dbg->add(normalized_original, "white margins");
+		}
+	}
+	
+	status.throwIfCancelled();
+
+	QColor bg_color(Qt::white);
+	if (!render_params.whiteMargins()) {
+		uint8_t const dominant_gray = reserveBlackAndWhite<uint8_t>(
+			calcDominantBackgroundGrayLevel(input.grayImage())
+		);
+		bg_color = QColor(dominant_gray, dominant_gray, dominant_gray);
+	}
+
+	QImage dewarped;
+	try {
+		dewarped = dewarp(
+			QTransform(), normalized_original, m_xform.transform(),
+			distortion_model, depth_perception, bg_color
+		);
+	} catch (std::runtime_error const&) {
+		// Probably an impossible distortion model.  Let's fall back to a trivial one.
+		setupTrivialDistortionModel(distortion_model);
+		dewarped = dewarp(
+			QTransform(), normalized_original, m_xform.transform(),
+			distortion_model, depth_perception, bg_color
+		);
+	}
+	normalized_original = QImage(); // Save memory.
+	if (dbg) {
+		dbg->add(dewarped, "dewarped");
+	}
+
+	status.throwIfCancelled();
+
+	QImage dewarped_and_maybe_smoothed;
+	// We only do smoothing if we are going to do binarization later.
+	if (!render_params.needBinarization()) {
+		dewarped_and_maybe_smoothed = dewarped;
+	} else {
+		dewarped_and_maybe_smoothed = smoothToGrayscale(dewarped, m_dpi);
+		if (dbg) {
+			dbg->add(dewarped_and_maybe_smoothed, "smoothed");
+		}
+	}
+
+	boost::shared_ptr<DewarpingPointMapper> mapper(
+		new DewarpingPointMapper(
+			distortion_model, depth_perception.value(),
+			m_xform.transform(), m_contentRect
+		)
+	);
+	boost::function<QPointF(QPointF const&)> const orig_to_output(
+		boost::bind(&DewarpingPointMapper::mapToDewarpedSpace, mapper, _1)
+	);
+
+	if (render_params.binaryOutput()) {	
+		BinaryImage dewarped_bw_content(dewarped_and_maybe_smoothed, bw_threshold);
+		dewarped_and_maybe_smoothed = QImage(); // Save memory.
+		if (dbg) {
+			dbg->add(dewarped_bw_content, "dewarped_bw_content");
+		}
+		
+		status.throwIfCancelled();
+
+		morphologicalSmoothInPlace(dewarped_bw_content, status);
+		if (dbg) {
+			dbg->add(dewarped_bw_content, "edges_smoothed");
+		}
+
+		status.throwIfCancelled();
+		
+		// It's important to keep despeckling the very last operation
+		// affecting the binary part of the output. That's because
+		// we will be reconstructing the input to this despeckling
+		// operation from the final output file.
+		maybeDespeckleInPlace(
+			dewarped_bw_content, m_outRect, m_outRect, m_despeckleLevel,
+			speckles_image, m_dpi, status, dbg
+		);
+
+		applyFillZonesInPlace(dewarped_bw_content, fill_zones, orig_to_output);
+
+		return dewarped_bw_content.toQImage();
+	}
+
+	if (!render_params.mixedOutput()) {
+		// It's "Color / Grayscale" mode, as we handle B/W above.
+		reserveBlackAndWhite(dewarped);
+	} else {
+		status.throwIfCancelled();
+
+		// Dewarp the B/W mask.
+		QTransform const orig_to_small_margins(
+			m_xform.transform() * QTransform().translate(
+				-small_margins_rect.left(),
+				-small_margins_rect.top()
+			)
+		);
+		QTransform small_margins_to_output;
+		small_margins_to_output.translate(
+			small_margins_rect.left(), small_margins_rect.top()
+		);
+		BinaryImage const dewarped_bw_mask(
+			dewarp(
+				orig_to_small_margins, warped_bw_mask.toQImage(),
+				small_margins_to_output, distortion_model,
+				depth_perception, Qt::black
+			)
+		);
+		if (dbg) {
+			dbg->add(dewarped_bw_mask, "dewarped_bw_mask");
+		}
+
+		status.throwIfCancelled();
+
+		BinaryImage dewarped_bw_content(dewarped_and_maybe_smoothed, bw_threshold);
+		dewarped_and_maybe_smoothed = QImage(); // Save memory.
+		if (dbg) {
+			dbg->add(dewarped_bw_content, "dewarped_bw_content");
+		}
+
+		status.throwIfCancelled();
+		
+		morphologicalSmoothInPlace(dewarped_bw_content, status);
+		if (dbg) {
+			dbg->add(dewarped_bw_content, "edges_smoothed");
+		}
+
+		status.throwIfCancelled();
+		
+		// We don't want speckles in non-B/W areas, as they would
+		// then get visualized on the Despeckling tab.
+		rasterOp<RopAnd<RopSrc, RopDst> >(dewarped_bw_content, dewarped_bw_mask);
+
+		status.throwIfCancelled();
+
+		// It's important to keep despeckling the very last operation
+		// affecting the binary part of the output. That's because
+		// we will be reconstructing the input to this despeckling
+		// operation from the final output file.
+		maybeDespeckleInPlace(
+			dewarped_bw_content, m_outRect, m_contentRect,
+			m_despeckleLevel, speckles_image, m_dpi, status, dbg
+		);
+		
+		status.throwIfCancelled();
+		
+		if (dewarped.format() == QImage::Format_Indexed8) {
+			combineMixed<uint8_t>(
+				dewarped, dewarped_bw_content, dewarped_bw_mask
+			);
+		} else {
+			assert(dewarped.format() == QImage::Format_RGB32
+				|| dewarped.format() == QImage::Format_ARGB32);
+			
+			combineMixed<uint32_t>(
+				dewarped, dewarped_bw_content, dewarped_bw_mask
+			);
+		}
+	}
+
+	applyFillZonesInPlace(dewarped, fill_zones, orig_to_output);
+
+	return dewarped;
+}
+
+/**
+ * Set up a distortion model corresponding to the content rect,
+ * which will result in no distortion correction.
+ */
+void
+OutputGenerator::setupTrivialDistortionModel(DistortionModel& distortion_model) const
+{
+	QPolygonF poly;
+	if (!m_contentRect.isEmpty()) {
+		poly = QRectF(m_contentRect);
+	} else {
+		poly << m_contentRect.topLeft() + QPointF(-0.5, -0.5);
+		poly << m_contentRect.topLeft() + QPointF(0.5, -0.5);
+		poly << m_contentRect.topLeft() + QPointF(0.5, 0.5);
+		poly << m_contentRect.topLeft() + QPointF(-0.5, 0.5);
+	}
+	poly = m_xform.transformBack().map(poly);
+	
+	std::vector<QPointF> top_polyline, bottom_polyline;
+	top_polyline.push_back(poly[0]); // top-left
+	top_polyline.push_back(poly[1]); // top-right
+	bottom_polyline.push_back(poly[3]); // bottom-left
+	bottom_polyline.push_back(poly[2]); // bottom-right
+	distortion_model.setTopCurve(Curve(top_polyline));
+	distortion_model.setBottomCurve(Curve(bottom_polyline));
+}
+
+CylindricalSurfaceDewarper
+OutputGenerator::createDewarper(
+	DistortionModel const& distortion_model,
+	QTransform const& distortion_model_to_target, double depth_perception)
+{
+	if (distortion_model_to_target.isIdentity()) {
+		return CylindricalSurfaceDewarper(
+			distortion_model.topCurve().polyline(),
+			distortion_model.bottomCurve().polyline(), depth_perception
+		);
+	}
+
+	std::vector<QPointF> top_polyline(distortion_model.topCurve().polyline());
+	std::vector<QPointF> bottom_polyline(distortion_model.bottomCurve().polyline());
+	BOOST_FOREACH(QPointF& pt, top_polyline) {
+		pt = distortion_model_to_target.map(pt);
+	}
+	BOOST_FOREACH(QPointF& pt, bottom_polyline) {
+		pt = distortion_model_to_target.map(pt);
+	}
+	return CylindricalSurfaceDewarper(
+		top_polyline, bottom_polyline, depth_perception
+	);
+}
+
+/**
+ * \param orig_to_src Transformation from the original image coordinates
+ *                    to the coordinate system of \p src image.
+ * \param src_to_output Transformation from the \p src image coordinates
+ *                      to output image coordinates.
+ * \param distortion_model Distortion model.
+ * \param depth_perception Depth perception.
+ * \param bg_color The color to use for areas outsize of \p src.
+ * \param modified_content_rect A vertically shrunk version of outputContentRect().
+ *                              See function definition for more details.
+ */
+QImage
+OutputGenerator::dewarp(
+	QTransform const& orig_to_src, QImage const& src,
+	QTransform const& src_to_output, DistortionModel const& distortion_model,
+	DepthPerception const& depth_perception, QColor const& bg_color) const
+{
+	CylindricalSurfaceDewarper const dewarper(
+		createDewarper(distortion_model, orig_to_src, depth_perception.value())
+	);
+
+	// Model domain is a rectangle in output image coordinates that
+	// will be mapped to our curved quadrilateral.
+	QRect const model_domain(
+		distortion_model.modelDomain(
+			dewarper, orig_to_src * src_to_output, outputContentRect()
+		).toRect()
+	);
+	if (model_domain.isEmpty()) {
+		GrayImage out(src.size());
+		out.fill(0xff); // white
+		return out;
+	}
+
+	return RasterDewarper::dewarp(
+		src, m_outRect.size(), dewarper, model_domain, bg_color
+	);
+}
+
 QSize
 OutputGenerator::from300dpi(QSize const& size, Dpi const& target_dpi)
 {
@@ -769,6 +1274,51 @@ OutputGenerator::to300dpi(QSize const& size, Dpi const& source_dpi)
 	return QSize(std::max(1, width), std::max(1, height));
 }
 
+QImage
+OutputGenerator::convertToRGBorRGBA(QImage const& src)
+{
+	QImage::Format const fmt = src.hasAlphaChannel()
+		? QImage::Format_ARGB32 : QImage::Format_RGB32;
+
+	return src.convertToFormat(fmt);
+}
+
+void
+OutputGenerator::fillMarginsInPlace(
+	QImage& image, QPolygonF const& content_poly, QColor const& color)
+{
+	if (image.format() == QImage::Format_Indexed8 && image.isGrayscale()) {
+		PolygonRasterizer::grayFillExcept(
+			image, qGray(color.rgb()), content_poly, Qt::WindingFill
+		);
+		return;
+	}
+
+	assert(image.format() == QImage::Format_RGB32 || image.format() == QImage::Format_ARGB32);
+
+	if (image.format() == QImage::Format_ARGB32) {
+		image = image.convertToFormat(QImage::Format_ARGB32_Premultiplied);
+	}
+
+	{
+		QPainter painter(&image);
+		painter.setRenderHint(QPainter::Antialiasing);
+		painter.setBrush(color);
+		painter.setPen(Qt::NoPen);
+
+		QPainterPath outer_path;
+		outer_path.addRect(image.rect());
+		QPainterPath inner_path;
+		inner_path.addPolygon(content_poly);
+
+		painter.drawPath(outer_path.subtracted(inner_path));
+	}
+
+	if (image.format() == QImage::Format_ARGB32_Premultiplied) {
+		image = image.convertToFormat(QImage::Format_ARGB32);
+	}
+}
+
 GrayImage
 OutputGenerator::detectPictures(
 	GrayImage const& input_300dpi, TaskStatus const& status,
@@ -878,6 +1428,36 @@ OutputGenerator::adjustThreshold(BinaryThreshold threshold) const
 	return BinaryThreshold(qBound(30, adjusted, 225));
 }
 
+BinaryThreshold
+OutputGenerator::calcBinarizationThreshold(
+	QImage const& image, BinaryImage const& mask) const
+{
+	GrayscaleHistogram hist(image, mask);
+	return adjustThreshold(BinaryThreshold::otsuThreshold(hist));
+}
+
+BinaryThreshold
+OutputGenerator::calcBinarizationThreshold(
+	QImage const& image, QPolygonF const& crop_area, BinaryImage const* mask) const
+{
+	QPainterPath path;
+	path.addPolygon(crop_area);
+	
+	if (path.contains(image.rect())) {
+		return adjustThreshold(BinaryThreshold::otsuThreshold(image));
+	} else {
+		BinaryImage modified_mask(image.size(), BLACK);
+		PolygonRasterizer::fillExcept(modified_mask, WHITE, crop_area, Qt::WindingFill);
+		modified_mask = erodeBrick(modified_mask, QSize(3, 3), WHITE);
+		
+		if (mask) {
+			rasterOp<RopAnd<RopSrc, RopDst> >(modified_mask, *mask);
+		}
+		
+		return calcBinarizationThreshold(image, modified_mask);
+	}
+}
+
 BinaryImage
 OutputGenerator::binarize(QImage const& image, BinaryImage const& mask) const
 {
@@ -947,10 +1527,10 @@ OutputGenerator::maybeDespeckleInPlace(
 	Dpi const& dpi, TaskStatus const& status, DebugImages* dbg) const
 {
 	QRect const src_rect(mask_rect.translated(-image_rect.topLeft()));
-	QRect const dst_rect(mask_rect.translated(-m_cropRect.topLeft()));
+	QRect const dst_rect(mask_rect);
 
 	if (speckles_img) {
-		BinaryImage(m_cropRect.size(), WHITE).swap(*speckles_img);
+		BinaryImage(m_outRect.size(), WHITE).swap(*speckles_img);
 		if (!mask_rect.isEmpty()) {
 			rasterOp<RopSrc>(*speckles_img, dst_rect, image, src_rect.topLeft());
 		}
@@ -1144,6 +1724,10 @@ OutputGenerator::calcLocalWindowSize(Dpi const& dpi)
 unsigned char
 OutputGenerator::calcDominantBackgroundGrayLevel(QImage const& img)
 {
+	// TODO: make a color version.
+	// In ColorPickupInteraction.cpp we have code for median color finding.
+	// We can use that.
+
 	QImage const gray(toGrayscale(img));
 	
 	BinaryImage mask(binarizeOtsu(gray));
@@ -1184,13 +1768,13 @@ OutputGenerator::calcDominantBackgroundGrayLevel(QImage const& img)
 
 void
 OutputGenerator::applyFillZonesInPlace(
-	QImage& img, ZoneSet const& zones) const
+	QImage& img, ZoneSet const& zones,
+	boost::function<QPointF(QPointF const&)> const& orig_to_output) const
 {
 	if (zones.empty()) {
 		return;
 	}
 
-	QTransform const xform(toOutput());
 	QImage canvas(img.convertToFormat(QImage::Format_ARGB32_Premultiplied));
 	
 	{
@@ -1200,7 +1784,7 @@ OutputGenerator::applyFillZonesInPlace(
 
 		BOOST_FOREACH(Zone const& zone, zones) {
 			QColor const color(zone.properties().locateOrDefault<FillColorProperty>()->color());
-			QPolygonF const poly(zone.spline().transformed(xform).toPolygon());
+			QPolygonF const poly(zone.spline().transformed(orig_to_output).toPolygon());
 			painter.setBrush(color);
 			painter.drawPolygon(poly, Qt::WindingFill);
 		}
@@ -1213,22 +1797,48 @@ OutputGenerator::applyFillZonesInPlace(
 	}
 }
 
+/**
+ * A simplified version of the above, using toOutput() for translation
+ * from original image to output image coordinates.
+ */
+void
+OutputGenerator::applyFillZonesInPlace(QImage& img, ZoneSet const& zones) const
+{
+	typedef QPointF (QTransform::*MapPointFunc)(QPointF const&) const;
+	applyFillZonesInPlace(
+		img, zones, boost::bind((MapPointFunc)&QTransform::map, m_xform.transform(), _1)
+	);
+}
+
 void
 OutputGenerator::applyFillZonesInPlace(
-	imageproc::BinaryImage& img, ZoneSet const& zones) const
+	imageproc::BinaryImage& img, ZoneSet const& zones,
+	boost::function<QPointF(QPointF const&)> const& orig_to_output) const
 {
 	if (zones.empty()) {
 		return;
 	}
 
-	QTransform const xform(toOutput());
-
 	BOOST_FOREACH(Zone const& zone, zones) {
 		QColor const color(zone.properties().locateOrDefault<FillColorProperty>()->color());
 		BWColor const bw_color = qGray(color.rgb()) < 128 ? BLACK : WHITE;
-		QPolygonF const poly(zone.spline().transformed(xform).toPolygon());
+		QPolygonF const poly(zone.spline().transformed(orig_to_output).toPolygon());
 		PolygonRasterizer::fill(img, bw_color, poly, Qt::WindingFill);
 	}
 }
 
+/**
+ * A simplified version of the above, using toOutput() for translation
+ * from original image to output image coordinates.
+ */
+void
+OutputGenerator::applyFillZonesInPlace(
+	imageproc::BinaryImage& img, ZoneSet const& zones) const
+{
+	typedef QPointF (QTransform::*MapPointFunc)(QPointF const&) const;
+	applyFillZonesInPlace(
+		img, zones, boost::bind((MapPointFunc)&QTransform::map, m_xform.transform(), _1)
+	);
+}
+
 } // namespace output
