@@ -97,7 +97,8 @@ public:
 	IntrusivePtr<CacheDrivenTask> createCacheDrivenTask(
 		IntrusivePtr<output::CacheDrivenTask> const& next_task);
 	
-	OptionsWidget* optionsWidget() { return m_ptrOptionsWidget.get(); }
+	OptionsWidget* optionsWidget() { return m_ptrOptionsWidget.get(); };
+	Settings* getSettings() { return m_ptrSettings.get(); };
 private:
 	void writePageSettings(
 		QDomDocument& doc, QDomElement& filter_el,
