@@ -21,6 +21,7 @@
 
 #include "NonCopyable.h"
 #include "RefCountable.h"
+#include "Margins.h"
 #include <memory>
 
 class PageId;
@@ -40,6 +41,8 @@ class Settings : public RefCountable
 public:
 	enum AggregateSizeChanged { AGGREGATE_SIZE_UNCHANGED, AGGREGATE_SIZE_CHANGED };
 	
+	static Margins defaultHardMarginsMM() { return Margins(10.0, 5.0, 10.0, 5.0); }
+
 	Settings();
 	
 	virtual ~Settings();
