@@ -38,6 +38,8 @@
 #include <QCoreApplication>
 #include <QDomElement>
 #include <stddef.h>
+#include "CommandLine.h"
+#include "OrderBySplitTypeProvider.h"
 
 namespace page_split
 {
@@ -45,13 +47,23 @@ namespace page_split
 Filter::Filter(IntrusivePtr<ProjectPages> const& page_sequence,
 	PageSelectionAccessor const& page_selection_accessor)
 :	m_ptrPages(page_sequence),
-	m_ptrSettings(new Settings)
+	m_ptrSettings(new Settings),
+	m_selectedPageOrder(0)
 {
-	m_ptrOptionsWidget.reset(
-		new OptionsWidget(
-			m_ptrSettings, m_ptrPages, page_selection_accessor
-		)
-	);
+	if (CommandLine::get().isGui()) {
+		m_ptrOptionsWidget.reset(
+			new OptionsWidget(
+				m_ptrSettings, m_ptrPages, page_selection_accessor
+			)
+		);
+	}
+	
+	typedef PageOrderOption::ProviderPtr ProviderPtr;
+
+	ProviderPtr const default_order;
+	ProviderPtr const order_by_splitline(new OrderBySplitTypeProvider(m_ptrSettings));
+	m_pageOrderOptions.push_back(PageOrderOption(tr("Natural order"), default_order));
+	m_pageOrderOptions.push_back(PageOrderOption(tr("Order by split type"), order_by_splitline));
 }
 
 Filter::~Filter()
@@ -215,4 +227,23 @@ Filter::createCacheDrivenTask(
 	);
 }
 
+std::vector<PageOrderOption>
+Filter::pageOrderOptions() const
+{
+	return m_pageOrderOptions;
+}
+
+int
+Filter::selectedPageOrder() const
+{
+	return m_selectedPageOrder;
+}
+
+void
+Filter::selectPageOrder(int option)
+{
+	assert((unsigned)option < m_pageOrderOptions.size());
+	m_selectedPageOrder = option;
+}
+
 } // page_split
