@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,13 +23,13 @@
 #include "RefCountable.h"
 #include "FilterResult.h"
 #include "IntrusivePtr.h"
-#include "ImageId.h"
+#include "PageInfo.h"
 #include <memory>
 
 class TaskStatus;
-class PageSequence;
 class FilterData;
 class DebugImages;
+class ProjectPages;
 class QImage;
 
 namespace deskew
@@ -42,6 +42,7 @@ namespace page_split
 
 class Filter;
 class Settings;
+class PageLayout;
 
 class Task : public RefCountable
 {
@@ -50,9 +51,9 @@ public:
 	Task(
 		IntrusivePtr<Filter> const& filter,
 		IntrusivePtr<Settings> const& settings,
-		IntrusivePtr<PageSequence> const& page_sequence,
+		IntrusivePtr<ProjectPages> const& pages,
 		IntrusivePtr<deskew::Task> const& next_task,
-		ImageId const& image_id,
+		PageInfo const& page_info,
 		bool batch_processing, bool debug);
 	
 	virtual ~Task();
@@ -60,13 +61,13 @@ public:
 	FilterResultPtr process(TaskStatus const& status, FilterData const& data);
 private:
 	class UiUpdater;
-	
+
 	IntrusivePtr<Filter> m_ptrFilter;
 	IntrusivePtr<Settings> m_ptrSettings;
-	IntrusivePtr<PageSequence> m_ptrPageSequence;
+	IntrusivePtr<ProjectPages> m_ptrPages;
 	IntrusivePtr<deskew::Task> m_ptrNextTask;
 	std::auto_ptr<DebugImages> m_ptrDbg;
-	ImageId m_imageId;
+	PageInfo m_pageInfo;
 	bool m_batchProcessing;
 };
 
