@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -189,4 +189,67 @@ PolygonUtils::fuzzyCompareImpl(QPointF const& p1, QPointF const& p2)
 	return dx <= ROUNDING_RECIP_MULTIPLIER && dy <= ROUNDING_RECIP_MULTIPLIER;
 }
 
+namespace
+{
+
+struct LexicographicPointComparator
+{
+	bool operator()(QPointF const& p1, QPointF const& p2) const {
+		if (p1.x() != p2.x()) {
+			return p1.x() < p2.x();
+		} else {
+			return p1.y() < p2.y();
+		}
+	}
+};
+
+// 2D cross product of OA and OB vectors, i.e. z-component of their 3D cross product.
+// Returns a positive value, if OAB makes a counter-clockwise turn,
+// negative for clockwise turn, and zero if the points are collinear.
+double cross(QPointF const& O, QPointF const& A, QPointF const& B)
+{
+	return (A.x() - O.x()) * (B.y() - O.y()) - (A.y() - O.y()) * (B.x() - O.x());
+}
+
+} // anonymous namespace
+
+QPolygonF
+PolygonUtils::convexHull(std::vector<QPointF> point_cloud)
+{
+	// "Monotone chain" algorithm.
+	// http://en.wikibooks.org/wiki/Algorithm_Implementation/Geometry/Convex_hull/Monotone_chain
+
+	int const n = point_cloud.size();
+	int k = 0;
+	std::vector<QPointF> hull(n * 2);
+
+	// Sort points by x, then y.
+	std::sort(point_cloud.begin(), point_cloud.end(), LexicographicPointComparator());
+
+    // Build lower hull.
+    for (int i = 0; i < n; ++i) {
+		while (k >= 2 && cross(hull[k-2], hull[k-1], point_cloud[i]) <= 0) {
+			k--;
+		}
+		hull[k++] = point_cloud[i];
+    }
+
+    // Build upper hull.
+    for (int i = n - 2, t = k + 1; i >= 0; --i) {
+		while (k >= t && cross(hull[k-2], hull[k-1], point_cloud[i]) <= 0) {
+			k--;
+		}
+        hull[k++] = point_cloud[i];
+    }
+
+    hull.resize(k);
+	
+	QPolygonF poly(k);
+	BOOST_FOREACH(QPointF const& pt, hull) {
+		poly << pt;
+	}
+    
+	return poly;
+}
+
 } // namespace imageproc
