@@ -20,17 +20,69 @@
 #define IMAGEPROC_TRANSFORM_H_
 
 #include <QSizeF>
+#include <QColor>
+#include <stdint.h>
 
 class QImage;
 class QRect;
 class QTransform;
-class QColor;
 
 namespace imageproc
 {
 
 class GrayImage;
 
+class OutsidePixels
+{
+	// Member-wise copying is OK.
+public:
+	enum Flags {
+		COLOR   = 1 << 0,
+		NEAREST = 1 << 1,
+		WEAK    = 1 << 2
+	};
+
+	/**
+	 * \brief Outside pixels are assumed to be of particular color.
+	 *
+	 * Outside pixels may be blended with inside pixels near the edges.
+	 */
+	static OutsidePixels assumeColor(QColor const& color) {
+		return OutsidePixels(COLOR, color.rgba());
+	}
+
+	/**
+	 * \brief Outside pixels are assumed to be of particular color.
+	 *
+	 * Outside pixels won't participate in blending operations.
+	 */
+	static OutsidePixels assumeWeakColor(QColor const& color) {
+		return OutsidePixels(WEAK|COLOR, color.rgba());
+	}
+
+	/**
+	 * \brief An outside pixel is assumed to be the same as the nearest inside pixel.
+	 *
+	 * Outside pixels won't participate in blending operations.
+	 */
+	static OutsidePixels assumeWeakNearest() {
+		return OutsidePixels(WEAK|NEAREST, 0xff000000);
+	}
+
+	int flags() const { return m_flags; }
+
+	QRgb rgba() const { return m_rgba; }
+
+	QRgb rgb() const { return m_rgba & 0x00ffff; }
+
+	uint8_t grayLevel() const { return static_cast<uint8_t>(qGray(m_rgba)); }
+private:
+	OutsidePixels(int flags, QRgb rgba) : m_flags(flags), m_rgba(rgba) {}
+
+	int m_flags;
+	QRgb m_rgba;
+};
+
 /**
  * \brief Apply an affine transformation to the image.
  *
@@ -57,8 +109,7 @@ class GrayImage;
  */
 QImage transform(
 	QImage const& src, QTransform const& xform,
-	QRect const& dst_rect, QColor const& background_color,
-	bool weak_background = false,
+	QRect const& dst_rect, OutsidePixels outside_pixels,
 	QSizeF const& min_mapping_area = QSizeF(0.9, 0.9));
 
 /**
@@ -69,8 +120,7 @@ QImage transform(
  */
 GrayImage transformToGray(
 	QImage const& src, QTransform const& xform,
-	QRect const& dst_rect, QColor const& background_color,
-	bool weak_background = false,
+	QRect const& dst_rect, OutsidePixels outside_pixels,
 	QSizeF const& min_mapping_area = QSizeF(0.9, 0.9));
 
 } // namespace imageproc
